@@ -75,6 +75,11 @@ struct _MetaScreen
   
   MetaWorkspace *active_workspace;
 
+  /* This window holds the focus when we don't want to focus
+   * any actual clients
+   */
+  Window no_focus_window;
+  
   GList *workspaces;
 
   MetaStack *stack;
@@ -141,6 +146,7 @@ MetaWorkspace* meta_screen_get_workspace_by_index (MetaScreen    *screen,
 
 void          meta_screen_set_cursor          (MetaScreen                 *screen,
                                                MetaCursor                  cursor);
+void          meta_screen_update_cursor       (MetaScreen                 *screen);
 
 void          meta_screen_ensure_tab_popup    (MetaScreen                 *screen,
                                                MetaTabList                 type);
