@@ -21,6 +21,7 @@
  */
 
 #include <config.h>
+#include "prefs.h"
 #include "ui.h"
 #include "frames.h"
 #include "util.h"
@@ -69,6 +70,7 @@ struct _EventFunc
 {
   MetaEventFunc func;
   gpointer data;
+  int last_event_serial;
 };
 
 static GdkFilterReturn
@@ -83,7 +85,10 @@ filter_func (GdkXEvent *xevent,
   if ((* ef->func) (xevent, ef->data))
     return GDK_FILTER_REMOVE;
   else
-    return GDK_FILTER_CONTINUE;
+    {
+      ef->last_event_serial = ((XEvent*)xevent)->xany.serial;
+      return GDK_FILTER_CONTINUE;
+    }
 }
 
 static EventFunc *ef = NULL;
@@ -116,6 +121,16 @@ meta_ui_remove_event_func (Display       *xdisplay,
   ef = NULL;
 }
 
+int
+meta_ui_get_last_event_serial (Display *xdisplay)
+{
+  g_assert (ef != NULL);
+
+  /* This is technically broken since it's not per-display */
+  
+  return ef->last_event_serial;
+}
+
 MetaUI*
 meta_ui_new (Display *xdisplay,
              Screen  *screen)
@@ -152,21 +167,92 @@ meta_ui_get_frame_geometry (MetaUI *ui,
                             left_width, right_width);
 }
 
-
-void
-meta_ui_add_frame (MetaUI *ui,
-                   Window  xwindow)
+Window
+meta_ui_create_frame_window (MetaUI *ui,
+                             Display *xdisplay,
+                             Visual *xvisual,
+			     gint x,
+			     gint y,
+			     gint width,
+			     gint height,
+			     gint screen_no)
 {
-  meta_frames_manage_window (ui->frames, xwindow);
+  GdkDisplay *display = gdk_x11_lookup_xdisplay (xdisplay);
+  GdkScreen *screen = gdk_display_get_screen (display, screen_no);
+  GdkWindowAttr attrs;
+  gint attributes_mask;
+  GdkWindow *window;
+  GdkVisual *visual;
+  GdkColormap *cmap = gdk_screen_get_default_colormap (screen);
+  
+  /* Default depth/visual handles clients with weird visuals; they can
+   * always be children of the root depth/visual obviously, but
+   * e.g. DRI games can't be children of a parent that has the same
+   * visual as the client.
+   */
+  if (!xvisual)
+    visual = gdk_screen_get_system_visual (screen);
+  else
+    {
+      visual = gdk_x11_screen_lookup_visual (screen,
+                                             XVisualIDFromVisual (xvisual));
+      cmap = gdk_colormap_new (visual, FALSE);
+    }
+
+  attrs.title = NULL;
+
+  /* frame.c is going to replace the event mask immediately, but
+   * we still have to set it here to let GDK know what it is.
+   */
+  attrs.event_mask =
+    GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
+    GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK |
+    GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK | GDK_FOCUS_CHANGE_MASK;
+  attrs.x = x;
+  attrs.y = y;
+  attrs.wclass = GDK_INPUT_OUTPUT;
+  attrs.visual = visual;
+  attrs.colormap = cmap;
+  attrs.window_type = GDK_WINDOW_CHILD;
+  attrs.cursor = NULL;
+  attrs.wmclass_name = NULL;
+  attrs.wmclass_class = NULL;
+  attrs.override_redirect = FALSE;
+
+  attrs.width  = width;
+  attrs.height = height;
+
+  attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
+
+  window =
+    gdk_window_new (gdk_screen_get_root_window(screen),
+		    &attrs, attributes_mask);
+
+  gdk_window_resize (window, width, height);
+  
+  meta_frames_manage_window (ui->frames, GDK_WINDOW_XID (window), window);
+
+  return GDK_WINDOW_XID (window);
 }
 
 void
-meta_ui_remove_frame (MetaUI *ui,
-                      Window  xwindow)
+meta_ui_destroy_frame_window (MetaUI *ui,
+			      Window  xwindow)
 {
   meta_frames_unmanage_window (ui->frames, xwindow);
 }
 
+void
+meta_ui_move_resize_frame (MetaUI *ui,
+			   Window frame,
+			   int x,
+			   int y,
+			   int width,
+			   int height)
+{
+  meta_frames_move_resize_frame (ui->frames, frame, x, y, width, height);
+}
+
 void
 meta_ui_map_frame   (MetaUI *ui,
                      Window  xwindow)
@@ -458,6 +544,7 @@ meta_gdk_pixbuf_get_from_pixmap (GdkPixbuf   *dest,
   GdkColormap *cmap;
   
   retval = NULL;
+  cmap = NULL;
   
   drawable = gdk_xid_table_lookup (xpixmap);
 
@@ -466,18 +553,21 @@ meta_gdk_pixbuf_get_from_pixmap (GdkPixbuf   *dest,
   else
     drawable = gdk_pixmap_foreign_new (xpixmap);
 
-  cmap = get_cmap (drawable);
+  if (drawable)
+    {
+      cmap = get_cmap (drawable);
   
-  retval = gdk_pixbuf_get_from_drawable (dest,
-                                         drawable,
-                                         cmap,
-                                         src_x, src_y,
-                                         dest_x, dest_y,
-                                         width, height);
-
+      retval = gdk_pixbuf_get_from_drawable (dest,
+                                             drawable,
+                                             cmap,
+                                             src_x, src_y,
+                                             dest_x, dest_y,
+                                             width, height);
+    }
   if (cmap)
     g_object_unref (G_OBJECT (cmap));
-  g_object_unref (G_OBJECT (drawable));
+  if (drawable)
+    g_object_unref (G_OBJECT (drawable));
 
   return retval;
 }
@@ -598,6 +688,44 @@ meta_text_property_to_utf8 (Display             *xdisplay,
   return retval;
 }
 
+void
+meta_ui_theme_get_frame_borders (MetaUI *ui,
+                                 MetaFrameType      type,
+                                 MetaFrameFlags     flags,
+                                 int               *top_height,
+                                 int               *bottom_height,
+                                 int               *left_width,
+                                 int               *right_width)
+{
+  int text_height;
+  PangoContext *context;
+  const PangoFontDescription *font_desc;
+  GtkStyle *default_style;
+
+  if (meta_ui_have_a_theme ())
+    {
+      context = gtk_widget_get_pango_context (GTK_WIDGET (ui->frames));
+      font_desc = meta_prefs_get_titlebar_font ();
+
+      if (!font_desc)
+        {
+          default_style = gtk_widget_get_default_style ();
+          font_desc = default_style->font_desc;
+        }
+
+      text_height = meta_pango_font_desc_get_text_height (font_desc, context);
+
+      meta_theme_get_frame_borders (meta_theme_get_current (),
+                                    type, text_height, flags,
+                                    top_height, bottom_height,
+                                    left_width, right_width);
+    }
+  else
+    {
+      *top_height = *bottom_height = *left_width = *right_width = 0;
+    }
+}
+
 void
 meta_ui_set_current_theme (const char *name,
                            gboolean    force_reload)
@@ -713,12 +841,11 @@ meta_ui_window_is_widget (MetaUI *ui,
 
   window = gdk_xid_table_lookup (xwindow);
 
-  if (window &&
-      gdk_window_get_window_type (window) != GDK_WINDOW_FOREIGN)
+  if (window)
     {
       void *user_data = NULL;
       gdk_window_get_user_data (window, &user_data);
-      return user_data != NULL;
+      return user_data != NULL && user_data != ui->frames;
     }
   else
     return FALSE;
@@ -759,7 +886,7 @@ meta_stock_icons_init (void)
       icon_set = gtk_icon_set_new_from_pixbuf (pixbuf);
       gtk_icon_factory_add (factory, items[i].stock_id, icon_set);
       gtk_icon_set_unref (icon_set);
-		
+      
       g_object_unref (G_OBJECT (pixbuf));
     }
 
