@@ -236,8 +236,8 @@ meta_workspace_relocate_windows (MetaWorkspace *workspace,
     {
       MetaWindow *window = tmp->data;
 
-      meta_workspace_add_window (new_home, window);
       meta_workspace_remove_window (workspace, window);
+      meta_workspace_add_window (new_home, window);
       
       tmp = tmp->next;
     }
@@ -847,6 +847,7 @@ meta_workspace_focus_default_window (MetaWorkspace *workspace,
                       "Setting focus to no_focus_window, since no valid "
                       "window to focus found.\n");
           meta_display_focus_the_no_focus_window (workspace->screen->display,
+                                                  workspace->screen,
                                                   timestamp);
         }
     }
@@ -910,7 +911,7 @@ focus_ancestor_or_mru_window (MetaWorkspace *workspace,
       MetaWindow* tmp_window;
       tmp_window = ((MetaWindow*) tmp->data);
       if (tmp_window != not_this_one           &&
-          !tmp_window->minimized               &&
+          meta_window_showing_on_its_workspace (tmp_window) &&
           tmp_window->type != META_WINDOW_DOCK &&
           tmp_window->type != META_WINDOW_DESKTOP)
         {
@@ -936,6 +937,7 @@ focus_ancestor_or_mru_window (MetaWorkspace *workspace,
     {
       meta_topic (META_DEBUG_FOCUS, "No MRU window to focus found; focusing no_focus_window.\n");
       meta_display_focus_the_no_focus_window (workspace->screen->display,
+                                              workspace->screen,
                                               timestamp);
     }
 }
